#
# Copyright (c) 2000 Persistent Systems Pvt Ltd (http://www.pspl.co.in/)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__doc__ = "Traversable Mixin class for ZClass"
__version__ = '0.2.0'

from Globals import HTMLFile

	
class TraversableMixin:
		""" A Traversable Mixin Class
		   
			Make this a base class of your ZClass to make them 'traversable'.
			A Traversable object is an object that can access components
			of the url as parameters.
                    
                   What does it mean?
                   
                        Consider the following URL:
                        
                        http://www.zopesite.com/MyTraversableMethod/foo/bar</code>
                        
                        In the usual scenario, zope would search for
                        *objects* called 'foo' and 'bar'.
                        
                        However, if 'MyTraversalMethod' is a traversable method,
                        it can *consume* components of the url as parameters. In
                        the above example, 'MyTraversalMethod' would have access
                        to the strings '"foo"' and '"bar"' which it can use
                        for any purpose.
                        
                        So where you earlier used:
                        
                        /Issues?priority=high&status=open
                        
                        you can now use:
                        
                        /Issues/HighPriority/Open
                        
                        Using this mixin class, you can make any other class
                        traversable. Works best with DTMLDocument and DTMLMethod.
                    
        
                Usage Notes - Creating the ZClass
                
                        - Always make this the _last_ base class for your ZClass
                        
                        - After creating a ZClass you need to add a view:
                        
                                1. Go to the 'Views' tab of the ZClass
                                
                                2. Type 'Traverse' for the view name
                                
                                3. Select 'manage_traverseform' from the dropdown
                                   list of available views
                                   
                        - The Mixin needs two properties to work. The names of 
                          these properties are 'mandatoryParameters' and 
                          'optionalParameters'.
                          You need to set them on each instance, before you
                          can use the traversal facilities on it. You should
                          tweak the ZClass_add methods to autmatically set
                          these properties (to 0) every time a new instance
                          is created.
                                   
                Usage Notes - Using the ZClass
                
                        - Specify the number of mandatory and optional paramters 
                          the method should consume.  Mandatory parameters are
                          *always consumed* from the url. Optional parameters
                          are consumed after that only if there is no object
                          that can be traversed with the same id.
                          
                          By default both the parameters are zero thus no url
                          components are consumed.
                        
                          Use the Traversal tab through the management interface
                          to specify the number of parameters.
                        
                        - Now in any method of your object you have a list available 
                          through the REQUEST['traverse_param'] variable.
                          
                          For example, if your object is a DTMLMethod, you can type
                          '<dtml-var traverse_param>' to see the list of parameters
                          consumed from the url.
                          
                Why the mandatory and optional parameters?
                
                        Consider a folder F with:
                        
                                - An object called 'foo'
                                
                                - A Traversable DTML Method called 'MyTMeth'
                        
                        Say you access the url '/F/MyTMeth/foo'. Should 'foo' be
                        interpreted as the acquired object or should it be used
                        as a parameter for 'MyTMeth'. To resolve this ambiguity
                        there is the concept of *mandatory* and *optional* 
                        parameters.

"""
		
		meta_type="Traversable Mixing Class"
		
		manage_options = (
			{'label': 'Traversal',	 'action': 'manage_traverseform'},
			)
			
		manage_traverseform = HTMLFile('traverseConfigForm', globals())
		
		
		def __bobo_traverse__(self, request, name):
			"""I'll take it from here, thanks (hee hee!)"""
			
			if (name[:6]=='manage') or (name[:2]=='aq'):
				return getattr(self,name)
			
			if (request.has_key('traverse_param')):
				numDone = len(request['traverse_param'])
			else:
				
				numDone = 0
				request['traverse_param'] = []
				
			if numDone < self.mandatoryParameters:
				(request['traverse_param']).append(name)
				return self
				
			else:
				if numDone < self.mandatoryParameters + self.optionalParameters:
					
				        if hasattr(self, name):
				                return getattr(self, name)
					else:
						(request['traverse_param']).append(name)
						return self
				else:
				        if hasattr(self, name):
				                return getattr(self, name)
				        raise 'Not Found', 'The requested resource was not found.'
						
				

		def manage_setTraverseOptions(self, REQUEST):
			"""Change the traversal options"""
			
			self.optionalParameters = int(REQUEST['optionalParameters'])
			self.mandatoryParameters = int(REQUEST['mandatoryParameters'])
			
			return self.manage_traverseform(self, REQUEST)
			